#!/usr/bin/env python
import glob
from Cheetah.Template import Template
from rdflib import Namespace, RDF
from rdflib.sparql import sparqlGraph
from rdflib.sparql import GraphPattern

DOAP = Namespace('http://usefulinc.com/ns/doap#')

class Products:
    def __init__(self):
        self._products = self._load()
        
    def __iter__(self):
        return self._products.iteritems()
   
    def _load(self):
        results = {}
        for product in glob.glob('products/*.n3'):
            product_name = product[product.index('/')+1:product.index('.')]
            if not product_name in results:
                results[product_name] = Product(product)
        return results

class Product:
    required = ('name', 'shortname', 'homepage', 'description', 'shortdesc', 
        'category', 'screenshots', 'programming_language', 'license', 
        'download_page', 'bug_database', 'mailing_list', 'wiki')
    
    def __init__(self, product):
        self._data = self._load(product)
        
    def __getitem__(self, key):
        return self._data[key]
        
    def _load(self, product):
        g = sparqlGraph.SPARQLGraph()
        g.load(product, format='n3')
        where = GraphPattern([('?x', RDF.type, DOAP['Project']), 
            ('?x', DOAP['name'], '?name'), 
            ('?x', DOAP['shortname'], '?shortname'), 
            ('?x', DOAP['homepage'], '?homepage'), 
            ('?x', DOAP['description'], '?description'), 
            ('?x', DOAP['shortdesc'], '?shortdesc'), 
            ('?x', DOAP['category'], '?category'),
            ('?x', DOAP['screenshots'], '?screenshots'),  
            ('?x', DOAP['programming-language'], '?programming_language'), 
            ('?x', DOAP['license'], '?license'), 
            ('?x', DOAP['download-page'], '?download_page'),
            ('?x', DOAP['bug-database'], '?bug_database'),
            ('?x', DOAP['mailing-list'], '?mailing_list'),
            ('?x', DOAP['wiki'], '?wiki')])
        query_result = g.query(['?'+i for i in self.required], where)
        results = {}
        i = 0
        for r in self.required:
            results[r] = query_result[0][i].encode('utf-8')
            i += 1
        return results

def products_list():
    products_list = {}
    for name, product in Products():
        if not products_list.has_key(product.__getitem__('category')): #wtf?
            products_list[product.__getitem__('category')] = []
        products_list[product.__getitem__('category')].append(product)
    fh = open('list.html', 'w')
    fh.write(str(Template(open('templates/list.html').read(), 
            [{'products_list': products_list}])))
        
    
def products_detail():
    for name, product in Products():
        fh = open('%s.html' % name, 'w')
        fh.write(str(Template(open('templates/detail.html').read(), 
            [{'product': product}])))

if __name__ == '__main__':
    products_list()
    products_detail()
        
        
